@@ -155,7 +155,7 @@ optDesign <- function(models, probs, doses,
       stop("n needs to be of length 1")
   }
   if(missing(Delta)){
-    if(substr(designCrit, 1, 3) == "TD")
+    if(designCrit %in% c("TD", "Dopt&TD"))
       stop("need to specify target difference \"Delta\"")
   } else {
     if(Delta <= 0)
@@ -183,14 +183,17 @@ optDesign <- function(models, probs, doses,
     stop("standDopt needs to contain a logical value")
   standInt <- as.integer(standDopt) # use standardized or non-stand. D-optimality
   nD <- length(doses)
-  if(designCrit == "TD" | designCrit == "TD&Dopt"){ # check whether TD exists in (0,max(dose))
+  if(designCrit == "TD" | designCrit == "Dopt&TD"){ # check whether TD exists in (0,max(dose))
+    if(length(unique(direction)) > 1)
+      stop("need to provide either \"increasing\" or \"decreasing\" as direction to optDesign, when TD optimal designs should be calculated")
+    direction <- unique(direction)
     tdMods <- TD(models, Delta, "continuous", direction)
     tdMods[tdMods > max(doses)] <- NA
     if(any(is.na(tdMods)))
       stop("TD does not exist for ",
            paste(names(tdMods)[is.na(tdMods)], collapse=", " ), " model(s)")
   }
-  if(designCrit == "Dopt" | designCrit == "TD&Dopt"){ # check whether Fisher matrix can be singular
+  if(designCrit == "Dopt" | designCrit == "Dopt&TD"){ # check whether Fisher matrix can be singular
     np <- nPars(names(models))
     if(max(np) > length(doses))
       stop("need at least as many dose levels as there are parameters to calculate Dopt design.")
@@ -268,7 +271,6 @@ optDesign <- function(models, probs, doses,
     }
   } else { # exact criterion (enumeration of all designs)
     ## enumerate possible exact designs
-    require(partitions, quietly = TRUE)
     con <- list(maxvls1 = 1e6, maxvls2 = 1e5, groupSize = 1)
     con[(namc <- names(control))] <- control    
     mat <- getDesMat(n, nD, lowbnd, uppbnd,
@@ -331,14 +333,17 @@ calcCrit <- function(design, models, probs, doses,
     stop("need to specify clinical relevance parameter")
   direction <- attr(models, "direction")
 
-  if(designCrit == "TD" | designCrit == "TD&Dopt"){ # check whether TD exists in (0,max(dose))
+  if(designCrit == "TD" | designCrit == "Dopt&TD"){ # check whether TD exists in (0,max(dose))
+    if(length(unique(direction)) > 1)
+      stop("need to provide either \"increasing\" or \"decreasing\" as direction to optDesign, when TD optimal designs should be calculated")
+    direction <- unique(direction)
     tdMods <- TD(models, Delta, "continuous", direction)
     tdMods[tdMods > max(doses)] <- NA
     if(any(is.na(tdMods)))
       stop("TD does not exist for ",
            paste(names(tdMods)[is.na(tdMods)], collapse=", " ), " model(s)")
   }
-  if(designCrit == "Dopt" | designCrit == "TD&Dopt"){ # check whether Fisher matrix can be singular
+  if(designCrit == "Dopt" | designCrit == "Dopt&TD"){ # check whether Fisher matrix can be singular
     np <- nPars(names(models))
     if(max(np) > length(doses))
       stop("need more dose levels to calculate Dopt design.")
@@ -413,6 +418,8 @@ which.is.max <- function (x){
 ## efficient rounding (see Pukelsheim (1993), Ch. 12)
 rndDesign <- function(design, n, eps = 0.0001){
 
+  if(missing(n))
+    stop("total sample size \"n\" needs to be specified")
   n <- round(n) # ensure n is an integer (at least numerically)
   if(inherits(design, "DRdesign")){
     design <- design$design
@@ -444,6 +451,16 @@ rndDesign <- function(design, n, eps = 0.0001){
   }
 }
 
+
+getCompositions <- function(N, M){
+  nC <- choose(N+M-1, M-1)
+  lst <- .C("getcomp", comp=integer(nC*M), integer(M-1),
+            as.integer(N), as.integer(M-1), as.integer(nC),
+            PACKAGE = "DoseFinding")
+  matrix(lst$comp, byrow = TRUE, nrow = nC)
+}
+
+
 ## calculate all possible compositions of n patients to nDoses groups
 ## (assuming a certain block-size) upper and lower bounds on the
 ## allocations can also be specified
@@ -458,7 +475,7 @@ getDesMat <- function(n, nDoses, lowbnd = rep(0, nDoses),
     stop(combn, " (unrestricted) combinations, increase maxvls1 in control 
          argument if this calculation should be performed")
 
-  desmat <- t(compositions(nG, nDoses))/nG
+  desmat <- getCompositions(nG, nDoses)/nG
  
   if(any(lowbnd > 0) | any(uppbnd < 1)){
     comp <- matrix(lowbnd, byrow = TRUE, ncol = nDoses, nrow=nrow(desmat))
